/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>

#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/FileSB.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/LabelG.h>
#include <Xm/Form.h>
#include <Xm/List.h>
#include <Xm/ScrollBar.h>
#include <X11/xpm.h>

#include "xmap.h"
#include "mode.h"
#include "debug.h"

extern Widget toplevel;
extern Widget draww;
extern GC clearGC;

static struct map *lmap;
static Dimension lwidth, lheight;

static double vlat, vlon, elat, elon;

static void drg_draw(int warp)
{
    Display *dpy = XtDisplay(draww);

    set_cursor(True);
    XFillRectangle(dpy, lmap->pmap, clearGC, 0, 0, lmap->mwidth, lmap->mheight);
    get_tiff_map(lmap, mapdata.mx_offset,
		 mapdata.my_offset,
		 lwidth, lheight, mapdata.zoom);
    refresh(warp);
    draw_scale();
    set_cursor(False);
}

do_drg(struct map *map)
{
    Pixmap new;
    XColor color;
    Colormap cmap = DefaultColormapOfScreen(XtScreen(draww));
    Display *dpy = XtDisplay(draww);

    lmap = map;
    lwidth = lheight = 0;
    color.red = (214 << 8);
    color.green = (227 << 8);
    color.blue = (255 << 8);
    myXAllocColor(dpy, cmap, &color);
    XSetForeground(dpy, clearGC, color.pixel);
}

draw_drg(int warp)
{
    Display *dpy = XtDisplay(draww);
    Dimension w, h;

    XtVaGetValues(draww,
		  XmNwidth, &w,
		  XmNheight, &h,
		  NULL);

    if (lwidth != w || lheight != h) {
	lwidth = w;
	lheight = h;
	if (lmap->pmap)
	    XFreePixmap(dpy, lmap->pmap);
	lmap->pmap = XCreatePixmap(dpy, XtWindow(draww), lwidth,
			 lheight, DefaultDepthOfScreen(XtScreen(draww)));
    }
    drg_draw(warp);
}

drg_center()
{
    draw_drg(TRUE);
}

void dzoomout()
{
    if (mapdata.zoom < max_un_zoom()) {
	mapdata.zoom++;
	mapdata.mx_offset -= lwidth / 2;
	mapdata.my_offset -= lheight / 2;
	if (mapdata.mx_offset < 0)
	    mapdata.mx_offset = 0;
	if (mapdata.my_offset < 0)
	    mapdata.my_offset = 0;
	drg_draw(FALSE);
    } else
	beep();
}

void dzoomout4()
{
    int zoomf;

    zoomf = max_un_zoom() - mapdata.zoom;
    if (zoomf) {
        if (zoomf > 4)
            zoomf = 4;
        mapdata.zoom += zoomf;
        mapdata.mx_offset -= (lwidth / 2 * zoomf);
        mapdata.my_offset -= (lheight / 2 * zoomf);
        if (mapdata.mx_offset < 0)
            mapdata.mx_offset = 0;
        if (mapdata.my_offset < 0)
            mapdata.my_offset = 0;
        drg_draw(FALSE);
    } else
        beep();
}


void dzoomin()
{
    if (mapdata.zoom > 1) {
	mapdata.zoom--;
	if (map.mwidth > (lwidth * mapdata.zoom))
	    mapdata.mx_offset += lwidth / 2;
	if (map.mheight > (lheight * mapdata.zoom))
	    mapdata.my_offset += lheight / 2;
	drg_draw(FALSE);
    } else
	beep();
}
