/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <X11/Xos.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/FileSB.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xmap.h"
#include "debug.h"

enum {
    READF, WRITEF, READR, WRITER, READM
};

extern XtAppContext app;
extern Widget toplevel;
static Widget dialog;
static Widget text_w;

static char *logfile;
static int what = 0;
static int done = 0;

void new_log_file(dialog, client_data, cbs)
Widget dialog;
XtPointer client_data;
XmFileSelectionBoxCallbackStruct *cbs;
{
    char *file = NULL;
    int status;

    XtUnmanageChild(dialog);
    if (cbs) {
	if (!XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &logfile)) {
	    done = 1;
	    return;		/* internal error */
	}
    }
    switch (what) {
    case WRITEF:
	new_log(logfile);
	break;
    case READF:
	read_log(logfile);
	break;
    case WRITER:
	route_save(logfile);
	break;
    case READR:
	route_read(logfile);
	break;
    case READM:
	marker_read(logfile);
	break;
    }
    done = 2;
}

void cancel(dialog, client_data, cbs)
Widget dialog;
XtPointer client_data;
XmFileSelectionBoxCallbackStruct *cbs;
{
    XtUnmanageChild(dialog);
    done = 1;
}


static void create_dialog()
{
    dialog = XmCreateFileSelectionDialog(toplevel, "file_sb", NULL, 0);

    /* Get the text widget handle stored as "user data" in File menu */
    XtAddCallback(dialog, XmNokCallback, new_log_file, NULL);
    XtAddCallback(dialog, XmNcancelCallback, (void *) XtUnmanageChild, NULL);
    XtManageChild(dialog);
}

static void real_select_log_file(Widget w, char *title, char *filter)
{
    XmString s0, s1;

    done = 0;

//fprintf(stderr,"in real_select_log_file(%d)\n",__LINE__);
    if (!dialog) {
	char *p;

//fprintf(stderr,"in real_select_log_file(%d)\n",__LINE__);
	create_dialog();
	p = getenv("HOME");

	if (p)
	    s0 = XmStringCreateSimple(p);
	XtVaSetValues(dialog,
		      XmNdirectory, s0,
		      NULL);
	if (p)
	    XmStringFree(s0);
    }
//fprintf(stderr,"in real_select_log_file(%d)\n",__LINE__);
    s0 = XmStringCreateSimple(title);
    s1 = XmStringCreateSimple(filter);
    XtVaSetValues(dialog,
		  XmNdialogTitle, s0,
		  XmNpattern, s1,
		  NULL);

//fprintf(stderr,"in real_select_log_file(%d)\n",__LINE__);
    XmStringFree(s0);
    XmStringFree(s1);
    XtManageChild(dialog);

//fprintf(stderr,"in real_select_log_file(%d)\n",__LINE__);
    XtPopup(XtParent(dialog), XtGrabNone);
    /* If dialog is already popped up, XtPopup does nothing.
     * Call XMapRaised() anyway to make sure it's visible.
     */
//fprintf(stderr,"in real_select_log_file(%d)\n",__LINE__);
    XMapRaised(XtDisplay(dialog), XtWindow(XtParent(dialog)));
//fprintf(stderr,"in real_select_log_file(%d)\n",__LINE__);
}

void select_log_file(Widget w)
{
    what = WRITEF;
    real_select_log_file(w, "Select a new log file", "*.xlog");
}

void select_marker_file(Widget w)
{
    what = READM;
    real_select_log_file(w, "Select a marker file", "*.xmar");
}

void select_ilog_file(Widget w)
{
    what = READF;
    real_select_log_file(w, "Select a log file", "*.xlog");
}

int select_route_file(Widget w, char *fn)
{
    what = WRITER;

    if (!fn)
	fn = "*.xrou";

    real_select_log_file(w, "Save route file", fn);
    while (!done)
	XtAppProcessEvent(app, XtIMAll);

    /* make sure the dialog goes away before returning. Sync with server
     * and update the display.
     */
    XSync(XtDisplay(dialog), 0);
    XmUpdateDisplay(toplevel);

    return done - 1;
}

void select_iroute_file(Widget w)
{
    what = READR;
//fprintf(stderr,"in select_iroute_file(%d)\n",__LINE__);
    real_select_log_file(w, "Select a route file", "*.xrou");
}
